/*
Copyright (c) 2002-2011 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
* Neither the name of Nokia Corporation nor the names of its contributors
 may be used to endorse or promote products derived from this software
 without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Description: 
*/ 

#ifndef OANDXCONTROLLER_H
#define OANDXCONTROLLER_H

#include <coecntrl.h>
#include <coemain.h>

#include <eikappui.h>
#include <eikapp.h>
#include <eikdoc.h>
#include <eikenv.h>
#include <eikon.hrh>
#include "oandxdefs.h"

class COandXEngine;

#define MaxInfoNoteTextLen 40

class COandXController : public CBase
/**
	This controller uses instructions from the user to update the
	engine (board.) 
 */
	{
public:
	static COandXController* NewL();
	virtual ~COandXController();

	// game control
	void Reset();
	TBool HitSquareL(TInt aIndex);
	void SwitchTurn();
	void ResetStats();

	// state
	enum TState
		{
		ENewGame, EPlaying, EFinished
		};

	inline TBool IsNewGame() const;
	inline TBool IsCrossTurn() const;
	inline TUint GamesPlayed() const;
	inline TUint WonByO() const;
	inline TUint WonByX() const;

	TTileState GameRecord(TInt aIndex);

	// stream persistence
	void ExternalizeL(RWriteStream& aStream) const;
	void InternalizeL(RReadStream& aStream);
	
private:
	void ConstructL();
	
private:
	// private persistent state
	TState iState;
	TBool iCrossTurn;
	
	TTileState iLastGameResult;
	TUint iNumGames;
	TUint iNumNoughtWins;
	TUint iNumCrossWins;
	TFixedArray<TTileState, KNumHistoryRecords> iGameRecords;
	};

// state inlines
inline TBool COandXController::IsNewGame() const { return iState==ENewGame; }
inline TBool COandXController::IsCrossTurn() const { return (iCrossTurn); }

inline TUint COandXController::GamesPlayed() const { return iNumGames; }
inline TUint COandXController::WonByO() const { return iNumNoughtWins; }
inline TUint COandXController::WonByX() const { return iNumCrossWins; }
	
#endif // OANDXCONTROLLER_H
